import openai

openai.api_key = ""

if __name__ == '__main__':
    file_path = "temp-part_1_5.mp3"
    # file_path = "temp_short_en.mp3"
    # file_path = "temp-part_1_1_en.mp3"
    audio_file = open(file_path, "rb")
    transcript = openai.Audio.transcribe("whisper-1", audio_file, response_format="text")
    print(transcript)



import re

import openai
import requests

from .util import chunk_mp3, PodcastData

class PodcastUtil:
    @staticmethod
    def convert_podcast_transcript(podcast_url):
        def download_mp3(url: str, file_path: str):
            with open(file_path, "wb") as file:
                response = requests.get(url)
                file.write(response.content)

        content = requests.get(podcast_url)
        mp3_url = re.findall("(?P<url>\;https?://[^\s]+)", content.text)[0].split(';')[1]
        print(f"mp3_url: {mp3_url}")
        mp3_file_path = "temp.mp3"
        download_mp3(mp3_url, mp3_file_path)
        print(f"Downloaded mp3 file")
        file_part_list = chunk_mp3(mp3_file_path)
        transcript_list = []
        for file_part in file_part_list:
            file = open(file_part, "rb")
            print(f"Calling openai whisper-1 for {file_part}")
            transcript = openai.Audio.transcribe("whisper-1", file)
            transcript_list.append(transcript)
        print(transcript_list)
        title = description = ""  # TODO
        return PodcastData(" ".join(transcript_list), title, description), True, "Success"



from .youtube_util import YoutubeUtil
from .podcast_util import PodcastUtil
from .pdf_util import PDFUtil
from .util import chunk_mp3, BaseData, YoutubeData, PodcastData
from .everything2text4prompt import Everything2Text4Prompt



class PDFUtil:
    @staticmethod
    def get_pdf_data(target_source):
        raise NotImplementedError("PDF to text conversion is not implemented yet")
        # with open(pdf_file, 'wb') as file:
        #     file.write(pdf_file)
        #
        # reader = PdfReader(pdf_file)
        # text = []
        #
        # for page in reader.pages:
        #     text.append(page.extract_text())
        #
        # return "\n".join(text)



import requests
from bs4 import BeautifulSoup
from youtube_transcript_api import YouTubeTranscriptApi

from .util import YoutubeData


class YoutubeUtil:
    @staticmethod
    def get_youtube_data(target_source: str) -> (str, bool, str):
        transcript = title = description = ts_transcript_list = None
        try:
            title, description = YoutubeUtil.get_video_info(target_source)
            transcript, ts_transcript_list = YoutubeUtil.convert_youtube_transcript(target_source)
        except Exception as e:
            return YoutubeData(transcript, title, description, ts_transcript_list), False, str(e)
        return YoutubeData(transcript, title, description, ts_transcript_list), True, ""

    @staticmethod
    def get_video_info(target_source):
        url = f"https://www.youtube.com/watch?v={target_source}"
        response = requests.get(url)

        if response.status_code != 200:
            return None

        soup = BeautifulSoup(response.content, "html.parser")

        title = soup.find("meta", itemprop="name")["content"]
        description = soup.find("meta", itemprop="description")["content"]

        return title, description

    @staticmethod
    def convert_youtube_transcript(target_source) -> (str, list):
        def extract_video_id_from_url(url):
            return url.split("v=")[1]

        def is_url(url):
            return "youtube.com" in url

        def merge_transcript(transcript):
            ts_transcript_list = []

            current_start = transcript[0]['start']
            current_text = transcript[0]['text']
            for i in range(1, len(transcript)):
                if transcript[i]['start'] - current_start >= 30:
                    ts_transcript_list.append({'text': current_text, 'start': current_start})
                    current_start = transcript[i]['start']
                    current_text = transcript[i]['text']
                else:
                    current_text += ' ' + transcript[i]['text']

            ts_transcript_list.append({'text': current_text, 'start': current_start})
            return ts_transcript_list

        if is_url(target_source):
            video_id = extract_video_id_from_url(target_source)
        else:
            video_id = target_source

        preferred_lang = ['en', 'zh', 'zh-Hans', 'zh-Hant', 'ja', 'it', 'de', 'fr']
        try:
            transcript = YouTubeTranscriptApi.get_transcript(video_id, preferred_lang)
        except Exception as e:
            import youtube_transcript_api
            if isinstance(e, youtube_transcript_api._errors.TranscriptsDisabled):
                raise Exception(e)
            transcript_list = YouTubeTranscriptApi.list_transcripts(video_id)
            transcript = transcript_list.find_generated_transcript(preferred_lang)
        text = " ".join([entry['text'] for entry in transcript])
        merged_transcript = merge_transcript(transcript)

        return text, merged_transcript


if __name__ == '__main__':
    target_source = 'lSTEhG021Jc'
    youtube_data, is_success, error_msg = YoutubeUtil.get_youtube_data(target_source)
    youtube_data: YoutubeData
    print(youtube_data.title)
    print(youtube_data.description)
    print(youtube_data.shorten_transcript)

    print("Transcript with timestamp:")
    # for entry in youtube_data.ts_transcript_list:
    #     print(f"{int(entry['start'] // 60)}:{int(entry['start'] % 60):02d} {entry['text'][:50]}")
    for entry in youtube_data.ts_transcript_list:
        print(f"{int(entry['start'] // 60)}:{int(entry['start'] % 60):02d} {entry['text']}")



import openai

from .pdf_util import PDFUtil
# from .podcast_util import convert_podcast_transcript
from .youtube_util import YoutubeUtil


class Everything2Text4Prompt:
    def __init__(self, openai_api_key, is_azure=False):
        self.openai_api_key = openai_api_key
        self.is_azure = is_azure
        openai.api_key = self.openai_api_key

    def convert_text(self, medium, target_source) -> (str, bool, str):
        if medium == "youtube":
            return YoutubeUtil.get_youtube_data(target_source)
        # elif medium == "podcast":
        #     return convert_podcast_transcript(target_source)
        elif medium == "pdf":
            return PDFUtil.get_pdf_data(target_source)
        else:
            raise Exception("Unsupported medium")


if __name__ == "__main__":
    openai_api_key = ""
    converter = Everything2Text4Prompt(openai_api_key)

    medium = "youtube"
    # target_source = "8S0FDjFBj8o"  # Default English
    target_source = "lSTEhG021Jc"  # Default auto-generated English
    # target_source = "https://www.youtube.com/watch?v=lSTEhG021Jc&ab_channel=EddieGM"  # Test the handling if people input URL
    # target_source = "https://www.youtube.com/watch?v=29WGNfuxIxc&ab_channel=PanSci%E6%B3%9B%E7%A7%91%E5%AD%B8"  # Default Chinese
    # target_source = "https://www.youtube.com/watch?v=K0SZ9mdygTw&t=757s&ab_channel=MuLi"  # Subtitle not available
    # target_source = "a"  # Error

    # medium = "podcast"
    # Short english
    # Moment 108 - This Powerful Tool Can Change Your Life: Africa Brooke
    # target_source = "https://podcasts.google.com/feed/aHR0cHM6Ly9mZWVkcy5idXp6c3Byb3V0LmNvbS8xNzE3MDMucnNz/episode/NWQzYmJlZDktNzA1Mi00NzU5LThjODctMzljMmIxNmJjZDM3?sa=X&ved=0CAUQkfYCahcKEwig_fW00YH_AhUAAAAAHQAAAAAQLA"

    # Long Chinese
    # TODO: Not sure why it is not working after chunking
    # 通用人工智能离我们多远，大模型专家访谈 ｜S7E11 硅谷徐老师 x OnBoard！
    # target_source = "https://podcasts.google.com/feed/aHR0cHM6Ly9mZWVkcy5maXJlc2lkZS5mbS9ndWlndXphb3poaWRhby9yc3M/episode/YzIxOWI4ZjktNTZiZi00NGQ3LTg3NjctYWZiNTQzOWZjMTNk?sa=X&ved=0CAUQkfYCahcKEwjwp9icjv_-AhUAAAAAHQAAAAAQLA&hl=zh-TW"

    data, is_success, error_msg = converter.convert_text(medium, target_source)
    print(data.shorten_transcript)
    print(is_success, error_msg)
    # print(data.ts_transcript_list)



import os

# Define the chunk size in bytes (20MB)
chunk_size = 20 * 1024 * 1024

# Open the input file in binary mode
file_path = "temp.mp3"
with open(file_path, "rb") as f:
    # Read the file as binary data
    data = f.read()

    # Calculate the number of chunks needed
    file_size = os.path.getsize(file_path)
    num_chunks = file_size // chunk_size + (1 if file_size % chunk_size != 0 else 0)

    # Split the data into chunks
    for i in range(num_chunks):
        start = i * chunk_size
        end = min((i + 1) * chunk_size, file_size)
        chunk_data = data[start:end]

        # Write the chunk to a new file
        with open(f"file-part{i}.mp3", "wb") as chunk_file:
            chunk_file.write(chunk_data)



import os


class BaseData:
    def __init__(self, full_content):
        self.full_content = full_content

    def shorten_text(self, text: str) -> str:
        if text is None or len(text) < 120:
            return text
        return text[:50] + " ... " + text[-50:]


class YoutubeData(BaseData):
    def __init__(self, transcript, title, description, ts_transcript_list):
        super().__init__(transcript)
        self.title = title
        self.description = description
        self.shorten_transcript = self.shorten_text(transcript)
        self.ts_transcript_list = ts_transcript_list


class PodcastData(BaseData):
    def __init__(self, transcript, title, description, ts_transcript_list):
        super().__init__(transcript)
        self.title = title
        self.description = description
        self.shorten_transcript = self.shorten_text(transcript)
        self.ts_transcript_list = ts_transcript_list


def chunk_mp3(file_path: str) -> list:
    chunk_size = 20 * 1024 * 1024  # 20MB

    with open(file_path, "rb") as f:
        data = f.read()  # Read the file as binary data

        # Calculate the number of chunks needed
        file_size = os.path.getsize(file_path)
        num_chunks = file_size // chunk_size + (1 if file_size % chunk_size != 0 else 0)

        # Split the data into chunks
        file_part_list = []
        for i in range(0, num_chunks):
            print(f"Splitting chunk {i + 1} of {num_chunks}")
            start = i * chunk_size
            end = min((i + 1) * chunk_size, file_size)
            chunk_data = data[start:end]

            # Write the chunk to a new file
            file_part = f"temp-part_{i + 1}_{num_chunks}_en.mp3"
            with open(file_part, "wb") as chunk_file:
                chunk_file.write(chunk_data)

            if os.path.getsize(file_part) < 2000:  # reject ~0 mp3 files
                continue
            file_part_list.append(file_part)
        print(f"chunk_mp3 summary: total_size = {file_size}, num_chunks = {num_chunks}")
    return file_part_list


if __name__ == '__main__':
    file_part_list = chunk_mp3("temp_short_en.mp3")
    print(file_part_list)



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="everything2text4prompt",
    version="0.0.8",
    description="Convert many medium into text, and the text format is specialized for prompt input",
    # package_dir={'': 'everything2text4prompt'},
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/michaelthwan/everything2text4prompt",
    author="michaethwan",
    author_email="michaelthwan@gmail.com",
    license="MIT",
    python_requires=">=3.8",
    install_requires=[
        "openai>=0.27.6",
        "youtube-transcript-api>=0.6.0",
        "bs4",
        "pypdf",
    ]
)
